part of spark_modal;

typedef SparkPickerBuilder = Widget Function(BuildContext context, int index);

typedef SparkPickerChanged = void Function(int index);

const String _sparkPickerTag = '_SPARK_PICKER';

typedef SparkPickerTitleBarBuilder = Widget Function(
  BuildContext context,
  Function cacel,
  Function confirm,
);

Future<Map<String, T>?> showSparkMultiplePicker<T>({
  required BuildContext context,
  required Map<String, List<SparkPickerData<T>>> options,
  double itemHeight = 40,
  SparkPickerBuilder? itemBuilder,
  bool showBar = true,
  String cancelText = '取消',
  String confirmText = '确定',
  TextStyle? cancelStyle,
  TextStyle? confirmStyle,
  double height = 300,
  SparkPickerTitleBarBuilder? barBuilder,
  Map<String, T>? defaultValue,
}) {
  return showSparkModalSheet(
      context: context,
      builder: (_ctx) {
        return _SparkMultiplePicker<T>(
          options: options,
          itemHeight: itemHeight,
          itemBuilder: itemBuilder,
          showBar: showBar,
          cancelStyle: cancelStyle,
          cancelText: cancelText,
          confirmStyle: confirmStyle,
          confirmText: confirmText,
          height: height,
          barBuilder: barBuilder,
          defaultValue: defaultValue,
          onConfirm: (_result) {
            Navigator.of(_ctx).pop(_result);
          },
          onCancel: () {
            Navigator.of(_ctx).pop();
          },
        );
      });
}

Future<T?> showSparkSingePicker<T>({
  required BuildContext context,
  required List<SparkPickerData<T>> options,
  double itemHeight = 40,
  SparkPickerBuilder? itemBuilder,
  bool showBar = true,
  String cancelText = '取消',
  String confirmText = '确定',
  TextStyle? cancelStyle,
  TextStyle? confirmStyle,
  double height = 300,
  SparkPickerTitleBarBuilder? barBuilder,
  T? defaultValue,
}) {
  return showSparkModalSheet<T>(
    context: context,
    shape: _defaultPickerShape,
    builder: (BuildContext _ctx) {
      return _SparkMultiplePicker<T>(
        options: {_sparkPickerTag: options},
        itemHeight: itemHeight,
        itemBuilder: itemBuilder,
        showBar: showBar,
        cancelStyle: cancelStyle,
        cancelText: cancelText,
        confirmStyle: confirmStyle,
        confirmText: confirmText,
        height: height,
        barBuilder: barBuilder,
        defaultValue:
            defaultValue != null ? {_sparkPickerTag: defaultValue} : null,
        onConfirm: (_result) {
          Navigator.of(_ctx).pop(_result[_sparkPickerTag]);
        },
        onCancel: () {
          Navigator.of(_ctx).pop();
        },
      );
    },
  );
}

class _SparkMultiplePicker<T> extends StatelessWidget {
  const _SparkMultiplePicker({
    required this.options,
    this.height = 300,
    this.itemHeight = 40,
    this.itemBuilder,
    this.showBar = true,
    this.cancelText = '取消',
    this.confirmText = '确定',
    this.cancelStyle,
    this.confirmStyle,
    this.barBuilder,
    this.defaultValue,
    this.onConfirm,
    this.onCancel,
    Key? key,
  }) : super(key: key);

  final Map<String, List<SparkPickerData<T>>> options;
  final double itemHeight;
  final SparkPickerBuilder? itemBuilder;
  final bool showBar;
  final String cancelText;
  final String confirmText;
  final TextStyle? cancelStyle;
  final TextStyle? confirmStyle;
  final double height;
  final SparkPickerTitleBarBuilder? barBuilder;
  final Function(Map<String, T>)? onConfirm;
  final Function()? onCancel;

  final Map<String, T>? defaultValue;

  Map<String, FixedExtentScrollController> getDefaultIndex() {
    Iterable<String> keys = options.keys;
    Map<String, FixedExtentScrollController> _data = {
      for (String _key in keys) _key: FixedExtentScrollController()
    };
    if (defaultValue == null) {
      return _data;
    }
    Iterable<MapEntry<String, T>> _entries = defaultValue!.entries;
    for (MapEntry<String, T> _element in _entries) {
      if (options.containsKey(_element.key)) {
        int _findIndex = options[_element.key]!
            .indexWhere((element) => element.value == _element.value);
        if (_findIndex != -1) {
          _data[_element.key] =
              FixedExtentScrollController(initialItem: _findIndex);
        }
      }
    }
    return _data;
  }

  void _onCancel() {
    if (onCancel != null) {
      onCancel!();
    }
  }

  void _onConfirm(Map<String, FixedExtentScrollController> raw) {
    Map<String, T> _data = <String, T>{};
    for (MapEntry<String, FixedExtentScrollController> _ in raw.entries) {
      _data[_.key] = options[_.key]![_.value.selectedItem].value;
    }
    if (onConfirm != null) {
      onConfirm!(_data);
    }
  }

  Widget? _buildBar(
      BuildContext context, Map<String, FixedExtentScrollController> raw) {
    if (!showBar) {
      return null;
    }
    if (barBuilder != null) {
      return barBuilder!(context, _onCancel, () {
        _onConfirm(raw);
      });
    }
    return Container(
      padding: const EdgeInsets.symmetric(horizontal: 14, vertical: 10),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          GestureDetector(
            child: Text(
              cancelText,
              style: cancelStyle ??
                  const TextStyle(
                    fontSize: 14,
                    color: Color(0xFF333333),
                  ),
            ),
            behavior: HitTestBehavior.opaque,
            onTap: _onCancel,
          ),
          GestureDetector(
            child: Text(
              confirmText,
              style: confirmStyle ??
                  const TextStyle(
                    fontSize: 14,
                    color: Color(0xFF2878FF),
                  ),
            ),
            behavior: HitTestBehavior.opaque,
            onTap: () => _onConfirm(raw),
          )
        ],
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    Map<String, FixedExtentScrollController> _indexRaw = getDefaultIndex();
    Widget? _bar = _buildBar(context, _indexRaw);
    return SizedBox(
      height: height,
      child: Column(
        children: [
          if (_bar != null) _bar,
          Expanded(
            child: Row(
              children: [
                for (MapEntry<String, FixedExtentScrollController> _entry
                    in _indexRaw.entries)
                  Expanded(
                    child: SparkPicker<T>(
                      options: options[_entry.key]!,
                      controller: _entry.value,
                      itemBuilder: itemBuilder,
                      itemHeight: itemHeight,
                    ),
                    flex: 1,
                  )
              ],
            ),
          )
        ],
      ),
    );
  }
}

class SparkPicker<T> extends StatelessWidget {
  const SparkPicker({
    required this.options,
    this.itemHeight = 40,
    this.itemBuilder,
    this.style,
    this.controller,
    Key? key,
  }) : super(key: key);

  final List<SparkPickerData<T>> options;

  final double itemHeight;

  final SparkPickerBuilder? itemBuilder;

  final TextStyle? style;

  final FixedExtentScrollController? controller;

  Widget _sparkPickerItemBuilder({
    required BuildContext context,
    required int index,
    required List<SparkPickerData> options,
    required double itemHeight,
  }) {
    return Container(
      height: itemHeight,
      alignment: Alignment.center,
      child: Text(
        options[index].label,
        style: style ?? const TextStyle(fontSize: 14, color: Color(0xFF333333)),
        textAlign: TextAlign.start,
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    FixedExtentScrollController _controller =
        controller ?? FixedExtentScrollController();
    return SparkCommonPicker(
      controller: _controller,
      itemHeight: itemHeight,
      itemBuilder: itemBuilder ??
          (_, _index) => _sparkPickerItemBuilder(
                context: _,
                itemHeight: itemHeight,
                options: options,
                index: _index,
              ),
      itemCount: options.length,
    );
  }
}

class SparkCommonPicker extends StatelessWidget {
  const SparkCommonPicker({
    Key? key,
    required this.controller,
    required this.itemHeight,
    required this.itemBuilder,
    required this.itemCount,
    this.onChanged,
    this.onChange,
    this.onStart,
    this.selectionOverlay,
  }) : super(key: key);

  /// picker controller
  final FixedExtentScrollController controller;

  /// single option height
  final double itemHeight;

  /// pick option builder
  final SparkPickerBuilder itemBuilder;

  /// item count
  final int itemCount;

  /// on changed, get last result
  final SparkPickerChanged? onChanged;

  /// scroll animation change index, get scrolling index value
  final SparkPickerChanged? onChange;

  final Function? onStart;

  /// selection overlay widget
  final Widget? selectionOverlay;

  void _selectedChangedWhenScrollEnd(int index) {
    if (onChanged != null) {
      onChanged!(index);
    }
  }

  void _selectedChangedWhenScrolling(int index) {
    if (onChange != null) {
      onChange!(index);
    }
  }

  @override
  Widget build(BuildContext context) {
    return NotificationListener(
      onNotification: (ScrollNotification notification) {
        if (notification.depth == 0 &&
            notification is ScrollEndNotification &&
            notification.metrics is FixedExtentMetrics) {
          final FixedExtentMetrics metrics =
              notification.metrics as FixedExtentMetrics;
          final int currentItemIndex = metrics.itemIndex;
          _selectedChangedWhenScrollEnd(currentItemIndex);
        }
        if (notification is ScrollStartNotification) {
          if (onStart != null) {
            onStart!();
          }
        }
        return false;
      },
      child: CupertinoPicker.builder(
        itemExtent: itemHeight,
        useMagnifier: true,
        selectionOverlay: selectionOverlay ??
            Container(
              decoration: const BoxDecoration(
                border: Border.symmetric(
                  horizontal: BorderSide(
                    color: Color(0xFFE5E5E5),
                  ),
                ),
              ),
            ),
        scrollController: controller,
        childCount: itemCount,
        itemBuilder: itemBuilder,
        onSelectedItemChanged: (int value) {
          _selectedChangedWhenScrolling(value);
        },
      ),
    );
  }
}

class SparkPickerData<T> {
  SparkPickerData.form(
    this.label,
    this.value,
    this.index,
  );

  /// show label
  String label;

  /// value
  T value;

  /// option index
  /// correspondence with the array
  int index;

  @override
  bool operator ==(Object other) {
    if (other is SparkPickerData) {
      return other.value == value && other.index == index;
    }
    return false;
  }

  @override
  int get hashCode => Object.hash(value, index, label);
}
